@@ -17,24 +17,32 @@ import hashlib
 import logging
 import os
 import time
+import typing as ty
 from urllib import parse as urlparse
-import warnings
+
+import requests
 
 from keystoneauth1 import _utils as utils
 from keystoneauth1 import access
 from keystoneauth1 import exceptions
 from keystoneauth1.identity.v3 import federation
+from keystoneauth1 import session as ks_session
 
 _logger = utils.get_logger(__name__)
 
-__all__ = ('OidcAuthorizationCode',
-           'OidcClientCredentials',
-           'OidcPassword',
-           'OidcAccessToken')
+__all__ = (
+    'OidcAuthorizationCode',
+    'OidcClientCredentials',
+    'OidcPassword',
+    'OidcAccessToken',
+)
 
 SENSITIVE_KEYS = ("password", "code", "token", "secret")
 
 
+_OidcBaseT = ty.TypeVar('_OidcBaseT', bound='_OidcBase')
+
+
 class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
     """Base class for different OpenID Connect based flows.
 
@@ -42,16 +50,31 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
     ``http://openid.net/specs/openid-connect-core-1_0.html``
     """
 
-    grant_type = None
+    grant_type: ty.ClassVar[str]
 
-    def __init__(self, auth_url, identity_provider, protocol,
-                 client_id, client_secret,
-                 access_token_type,
-                 scope="openid profile",
-                 access_token_endpoint=None,
-                 discovery_endpoint=None,
-                 grant_type=None,
-                 **kwargs):
+    def __init__(
+        self,
+        auth_url: str,
+        identity_provider: str,
+        protocol: str,
+        client_id: str,
+        client_secret: ty.Optional[str],
+        access_token_type: str,
+        scope: str,
+        access_token_endpoint: ty.Optional[str],
+        discovery_endpoint: ty.Optional[str],
+        *,
+        trust_id: ty.Optional[str],
+        system_scope: ty.Optional[str],
+        domain_id: ty.Optional[str],
+        domain_name: ty.Optional[str],
+        project_id: ty.Optional[str],
+        project_name: ty.Optional[str],
+        project_domain_id: ty.Optional[str],
+        project_domain_name: ty.Optional[str],
+        reauthenticate: bool,
+        include_catalog: bool,
+    ):
         """The OpenID Connect plugin expects the following.
 
         :param auth_url: URL of the Identity Service
@@ -86,8 +109,8 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         :type access_token_endpoint: string
 
         :param discovery_endpoint: OpenID Connect Discovery Document URL,
-                                   for example:
-                  https://localhost:8020/oidc/.well-known/openid-configuration
+            for example:
+            https://localhost:8020/oidc/.well-known/openid-configuration
         :type access_token_endpoint: string
 
         :param scope: OpenID Connect scope that is requested from OP,
@@ -96,32 +119,35 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
                       states that "openid" must be always specified.
         :type scope: string
         """
-        super(_OidcBase, self).__init__(auth_url, identity_provider, protocol,
-                                        **kwargs)
+        super().__init__(
+            auth_url=auth_url,
+            identity_provider=identity_provider,
+            protocol=protocol,
+            trust_id=trust_id,
+            system_scope=system_scope,
+            domain_id=domain_id,
+            domain_name=domain_name,
+            project_id=project_id,
+            project_name=project_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name,
+            reauthenticate=reauthenticate,
+            include_catalog=include_catalog,
+        )
+
         self.client_id = client_id
         self.client_secret = client_secret
 
         self.discovery_endpoint = discovery_endpoint
-        self._discovery_document = {}
+        self._discovery_document: dict[str, object] = {}
         self.access_token_endpoint = access_token_endpoint
 
         self.access_token_type = access_token_type
         self.scope = scope
 
-        if grant_type is not None:
-            if grant_type != self.grant_type:
-                raise exceptions.OidcGrantTypeMissmatch()
-            warnings.warn("Passing grant_type as an argument has been "
-                          "deprecated as it is now defined in the plugin "
-                          "itself. You should stop passing this argument "
-                          "to the plugin, as it will be ignored, since you "
-                          "cannot pass a free text string as a grant_type. "
-                          "This argument will be dropped from the plugin in "
-                          "July 2017 or with the next major release of "
-                          "keystoneauth (3.0.0)",
-                          DeprecationWarning)
-
-    def _get_discovery_document(self, session):
+    def _get_discovery_document(
+        self, session: ks_session.Session
+    ) -> dict[str, object]:
         """Get the contents of the OpenID Connect Discovery Document.
 
         This method grabs the contents of the OpenID Connect Discovery Document
@@ -137,14 +163,18 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
                   otherwise it will return an empty dict.
         :rtype: dict
         """
-        if (self.discovery_endpoint is not None
-                and not self._discovery_document):
+        if (
+            self.discovery_endpoint is not None
+            and not self._discovery_document
+        ):
             try:
-                resp = session.get(self.discovery_endpoint,
-                                   authenticated=False)
+                resp = session.get(
+                    self.discovery_endpoint, authenticated=False
+                )
             except exceptions.HttpError:
-                _logger.error("Cannot fetch discovery document %(discovery)s" %
-                              {"discovery": self.discovery_endpoint})
+                _logger.error(
+                    f"Cannot fetch discovery document {self.discovery_endpoint}"
+                )
                 raise
 
             try:
@@ -157,7 +187,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
 
         return self._discovery_document
 
-    def _get_access_token_endpoint(self, session):
+    def _get_access_token_endpoint(self, session: ks_session.Session) -> str:
         """Get the "token_endpoint" for the OpenID Connect flow.
 
         This method will return the correct access token endpoint to be used.
@@ -170,7 +200,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         :type session: keystoneauth1.session.Session
 
         :return: the endpoint to use
-        :rtype: string or None if no endpoint is found
+        :rtype: string
         """
         if self.access_token_endpoint is not None:
             return self.access_token_endpoint
@@ -179,25 +209,34 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         endpoint = discovery.get("token_endpoint")
         if endpoint is None:
             raise exceptions.OidcAccessTokenEndpointNotFound()
-        return endpoint
+        return ty.cast(str, endpoint)
 
-    def _sanitize(self, data):
+    def _sanitize(
+        self, data: dict[str, ty.Optional[str]]
+    ) -> dict[str, ty.Optional[str]]:
         sanitized = copy.deepcopy(data)
         for key in sanitized:
             if any(s in key for s in SENSITIVE_KEYS):
                 sanitized[key] = "***"
         return sanitized
 
-    def _get_access_token(self, session, payload):
+    def _get_access_token(
+        self, session: ks_session.Session, payload: dict[str, ty.Optional[str]]
+    ) -> str:
         """Exchange a variety of user supplied values for an access token.
 
         :param session: a session object to send out HTTP requests.
         :type session: keystoneauth1.session.Session
 
         :param payload: a dict containing various OpenID Connect values, for
-                        example::
-                          {'grant_type': 'password', 'username': self.username,
-                           'password': self.password, 'scope': self.scope}
+            example::
+
+                {
+                    'grant_type': 'password',
+                    'username': self.username,
+                    'password': self.password,
+                    'scope': self.scope,
+                }
         :type payload: dict
         """
         if self.client_secret:
@@ -211,24 +250,33 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
             sanitized_payload = self._sanitize(payload)
             _logger.debug(
                 "Making OpenID-Connect authentication request to %s with "
-                "data %s", access_token_endpoint, sanitized_payload
+                "data %s",
+                access_token_endpoint,
+                sanitized_payload,
             )
 
-        op_response = session.post(access_token_endpoint,
-                                   requests_auth=client_auth,
-                                   data=payload,
-                                   log=False,
-                                   authenticated=False)
+        op_response = session.post(
+            access_token_endpoint,
+            requests_auth=client_auth,
+            data=payload,
+            log=False,
+            authenticated=False,
+        )
         response = op_response.json()
         if _logger.isEnabledFor(logging.DEBUG):
             sanitized_response = self._sanitize(response)
             _logger.debug(
                 "OpenID-Connect authentication response from %s is %s",
-                access_token_endpoint, sanitized_response
+                access_token_endpoint,
+                sanitized_response,
             )
-        return response[self.access_token_type]
+        access_token = response[self.access_token_type]
+        assert isinstance(access_token, str)  # nosec: B101
+        return access_token
 
-    def _get_keystone_token(self, session, access_token):
+    def _get_keystone_token(
+        self, session: ks_session.Session, access_token: str
+    ) -> requests.Response:
         r"""Exchange an access token for a keystone token.
 
         By Sending the access token in an `Authorization: Bearer` header, to
@@ -247,12 +295,14 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         """
         # use access token against protected URL
         headers = {'Authorization': 'Bearer ' + access_token}
-        auth_response = session.post(self.federated_token_url,
-                                     headers=headers,
-                                     authenticated=False)
+        auth_response = session.post(
+            self.federated_token_url, headers=headers, authenticated=False
+        )
         return auth_response
 
-    def get_unscoped_auth_ref(self, session):
+    def get_unscoped_auth_ref(
+        self, session: ks_session.Session
+    ) -> access.AccessInfoV3:
         """Authenticate with OpenID Connect and get back claims.
 
         This is a multi-step process:
@@ -277,9 +327,14 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         """
         # First of all, check if the grant type is supported
         discovery = self._get_discovery_document(session)
-        grant_types = discovery.get("grant_types_supported")
-        if (grant_types and self.grant_type is not None
-                and self.grant_type not in grant_types):
+        grant_types = ty.cast(
+            ty.Optional[list[str]], discovery.get("grant_types_supported")
+        )
+        if (
+            grant_types
+            and self.grant_type is not None
+            and self.grant_type not in grant_types
+        ):
             raise exceptions.OidcPluginNotSupported()
 
         # Get the payload
@@ -292,10 +347,14 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         response = self._get_keystone_token(session, access_token)
 
         # grab the unscoped token
-        return access.create(resp=response)
+        access_info = access.create(resp=response)
+        assert isinstance(access_info, access.AccessInfoV3)  # nosec B101
+        return access_info
 
     @abc.abstractmethod
-    def get_payload(self, session):
+    def get_payload(
+        self, session: ks_session.Session
+    ) -> dict[str, ty.Optional[str]]:
         """Get the plugin specific payload for obtainin an access token.
 
         OpenID Connect supports different grant types. This method should
@@ -317,13 +376,32 @@ class OidcPassword(_OidcBase):
 
     grant_type = "password"
 
-    def __init__(self, auth_url, identity_provider, protocol,  # nosec
-                 client_id, client_secret,
-                 access_token_endpoint=None,
-                 discovery_endpoint=None,
-                 access_token_type='access_token',
-                 username=None, password=None, idp_otp_key=None,
-                 **kwargs):
+    def __init__(
+        self,
+        auth_url: str,
+        identity_provider: str,
+        protocol: str,
+        client_id: str,
+        client_secret: str,
+        access_token_type: str = 'access_token',  # nosec B107
+        scope: str = 'openid profile',
+        access_token_endpoint: ty.Optional[str] = None,
+        discovery_endpoint: ty.Optional[str] = None,
+        username: ty.Optional[str] = None,
+        password: ty.Optional[str] = None,
+        idp_otp_key: ty.Optional[str] = None,
+        *,
+        trust_id: ty.Optional[str] = None,
+        system_scope: ty.Optional[str] = None,
+        domain_id: ty.Optional[str] = None,
+        domain_name: ty.Optional[str] = None,
+        project_id: ty.Optional[str] = None,
+        project_name: ty.Optional[str] = None,
+        project_domain_id: ty.Optional[str] = None,
+        project_domain_name: ty.Optional[str] = None,
+        reauthenticate: bool = True,
+        include_catalog: bool = True,
+    ):
         """The OpenID Password plugin expects the following.
 
         :param username: Username used to authenticate
@@ -332,21 +410,34 @@ class OidcPassword(_OidcBase):
         :param password: Password used to authenticate
         :type password: string
         """
-        super(OidcPassword, self).__init__(
+        super().__init__(
             auth_url=auth_url,
             identity_provider=identity_provider,
             protocol=protocol,
             client_id=client_id,
             client_secret=client_secret,
+            access_token_type=access_token_type,
+            scope=scope,
             access_token_endpoint=access_token_endpoint,
             discovery_endpoint=discovery_endpoint,
-            access_token_type=access_token_type,
-            **kwargs)
+            trust_id=trust_id,
+            system_scope=system_scope,
+            domain_id=domain_id,
+            domain_name=domain_name,
+            project_id=project_id,
+            project_name=project_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name,
+            reauthenticate=reauthenticate,
+            include_catalog=include_catalog,
+        )
         self.username = username
         self.password = password
         self.idp_otp_key = idp_otp_key
 
-    def get_payload(self, session):
+    def get_payload(
+        self, session: ks_session.Session
+    ) -> dict[str, ty.Optional[str]]:
         """Get an authorization grant for the "password" grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -355,16 +446,20 @@ class OidcPassword(_OidcBase):
         :returns: a python dictionary containing the payload to be exchanged
         :rtype: dict
         """
-        payload = {'username': self.username,
-                   'password': self.password,
-                   'scope': self.scope,
-                   'client_id': self.client_id}
+        payload = {
+            'username': self.username,
+            'password': self.password,
+            'scope': self.scope,
+            'client_id': self.client_id,
+        }
 
         self.manage_otp_from_session_or_request_to_the_user(payload, session)
 
         return payload
 
-    def manage_otp_from_session_or_request_to_the_user(self, payload, session):
+    def manage_otp_from_session_or_request_to_the_user(
+        self, payload: dict[str, ty.Optional[str]], session: ks_session.Session
+    ) -> None:
         """Get the OTP code from the session or else request to the user.
 
         When the OS_IDP_OTP_KEY environment variable is set, this method will
@@ -391,7 +486,8 @@ class OidcPassword(_OidcBase):
             payload[self.idp_otp_key] = otp_from_session
         else:
             payload[self.idp_otp_key] = input(
-                "Please, enter the generated OTP code: ")
+                "Please, enter the generated OTP code: "
+            )
             setattr(session, 'otp', payload[self.idp_otp_key])
 
 
@@ -400,12 +496,29 @@ class OidcClientCredentials(_OidcBase):
 
     grant_type = 'client_credentials'
 
-    def __init__(self, auth_url, identity_provider, protocol,  # nosec
-                 client_id, client_secret,
-                 access_token_endpoint=None,
-                 discovery_endpoint=None,
-                 access_token_type='access_token',
-                 **kwargs):
+    def __init__(
+        self,
+        auth_url: str,
+        identity_provider: str,
+        protocol: str,
+        client_id: str,
+        client_secret: str,
+        access_token_type: str = 'access_token',  # nosec B107
+        scope: str = 'openid profile',
+        access_token_endpoint: ty.Optional[str] = None,
+        discovery_endpoint: ty.Optional[str] = None,
+        *,
+        trust_id: ty.Optional[str] = None,
+        system_scope: ty.Optional[str] = None,
+        domain_id: ty.Optional[str] = None,
+        domain_name: ty.Optional[str] = None,
+        project_id: ty.Optional[str] = None,
+        project_name: ty.Optional[str] = None,
+        project_domain_id: ty.Optional[str] = None,
+        project_domain_name: ty.Optional[str] = None,
+        reauthenticate: bool = True,
+        include_catalog: bool = True,
+    ):
         """The OpenID Client Credentials expects the following.
 
         :param client_id: Client ID used to authenticate
@@ -414,18 +527,31 @@ class OidcClientCredentials(_OidcBase):
         :param client_secret: Client Secret used to authenticate
         :type password: string
         """
-        super(OidcClientCredentials, self).__init__(
+        super().__init__(
             auth_url=auth_url,
             identity_provider=identity_provider,
             protocol=protocol,
             client_id=client_id,
             client_secret=client_secret,
+            access_token_type=access_token_type,
+            scope=scope,
             access_token_endpoint=access_token_endpoint,
             discovery_endpoint=discovery_endpoint,
-            access_token_type=access_token_type,
-            **kwargs)
+            trust_id=trust_id,
+            system_scope=system_scope,
+            domain_id=domain_id,
+            domain_name=domain_name,
+            project_id=project_id,
+            project_name=project_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name,
+            reauthenticate=reauthenticate,
+            include_catalog=include_catalog,
+        )
 
-    def get_payload(self, session):
+    def get_payload(
+        self, session: ks_session.Session
+    ) -> dict[str, ty.Optional[str]]:
         """Get an authorization grant for the client credentials grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -434,7 +560,7 @@ class OidcClientCredentials(_OidcBase):
         :returns: a python dictionary containing the payload to be exchanged
         :rtype: dict
         """
-        payload = {'scope': self.scope}
+        payload: dict[str, ty.Optional[str]] = {'scope': self.scope}
         return payload
 
 
@@ -443,12 +569,31 @@ class OidcAuthorizationCode(_OidcBase):
 
     grant_type = 'authorization_code'
 
-    def __init__(self, auth_url, identity_provider, protocol,  # nosec
-                 client_id, client_secret,
-                 access_token_endpoint=None,
-                 discovery_endpoint=None,
-                 access_token_type='access_token',
-                 redirect_uri=None, code=None, **kwargs):
+    def __init__(
+        self,
+        auth_url: str,
+        identity_provider: str,
+        protocol: str,
+        client_id: str,
+        client_secret: str,
+        access_token_type: str = 'access_token',  # nosec B107
+        scope: str = 'openid profile',
+        access_token_endpoint: ty.Optional[str] = None,
+        discovery_endpoint: ty.Optional[str] = None,
+        code: ty.Optional[str] = None,
+        *,
+        trust_id: ty.Optional[str] = None,
+        system_scope: ty.Optional[str] = None,
+        domain_id: ty.Optional[str] = None,
+        domain_name: ty.Optional[str] = None,
+        project_id: ty.Optional[str] = None,
+        project_name: ty.Optional[str] = None,
+        project_domain_id: ty.Optional[str] = None,
+        project_domain_name: ty.Optional[str] = None,
+        reauthenticate: bool = True,
+        include_catalog: bool = True,
+        redirect_uri: ty.Optional[str] = None,
+    ):
         """The OpenID Authorization Code plugin expects the following.
 
         :param redirect_uri: OpenID Connect Client Redirect URL
@@ -458,20 +603,33 @@ class OidcAuthorizationCode(_OidcBase):
         :type code: string
 
         """
-        super(OidcAuthorizationCode, self).__init__(
+        super().__init__(
             auth_url=auth_url,
             identity_provider=identity_provider,
             protocol=protocol,
             client_id=client_id,
             client_secret=client_secret,
+            access_token_type=access_token_type,
+            scope=scope,
             access_token_endpoint=access_token_endpoint,
             discovery_endpoint=discovery_endpoint,
-            access_token_type=access_token_type,
-            **kwargs)
+            trust_id=trust_id,
+            system_scope=system_scope,
+            domain_id=domain_id,
+            domain_name=domain_name,
+            project_id=project_id,
+            project_name=project_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name,
+            reauthenticate=reauthenticate,
+            include_catalog=include_catalog,
+        )
         self.redirect_uri = redirect_uri
         self.code = code
 
-    def get_payload(self, session):
+    def get_payload(
+        self, session: ks_session.Session
+    ) -> dict[str, ty.Optional[str]]:
         """Get an authorization grant for the "authorization_code" grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -488,8 +646,30 @@ class OidcAuthorizationCode(_OidcBase):
 class OidcAccessToken(_OidcBase):
     """Implementation for OpenID Connect access token reuse."""
 
-    def __init__(self, auth_url, identity_provider, protocol,
-                 access_token, **kwargs):
+    def __init__(
+        self,
+        auth_url: str,
+        identity_provider: str,
+        protocol: str,
+        # client_id and client_id intentionally omitted since they don't make
+        # sense with an access token
+        access_token_type: str = 'access_token',  # nosec B107
+        scope: str = 'openid profile',
+        access_token_endpoint: ty.Optional[str] = None,
+        discovery_endpoint: ty.Optional[str] = None,
+        access_token: ty.Optional[str] = None,
+        *,
+        trust_id: ty.Optional[str] = None,
+        system_scope: ty.Optional[str] = None,
+        domain_id: ty.Optional[str] = None,
+        domain_name: ty.Optional[str] = None,
+        project_id: ty.Optional[str] = None,
+        project_name: ty.Optional[str] = None,
+        project_domain_id: ty.Optional[str] = None,
+        project_domain_name: ty.Optional[str] = None,
+        reauthenticate: bool = True,
+        include_catalog: bool = True,
+    ):
         """The OpenID Connect plugin based on the Access Token.
 
         It expects the following:
@@ -507,20 +687,41 @@ class OidcAccessToken(_OidcBase):
         :param access_token: OpenID Connect Access token
         :type access_token: string
         """
-        super(OidcAccessToken, self).__init__(auth_url, identity_provider,
-                                              protocol,
-                                              client_id=None,
-                                              client_secret=None,
-                                              access_token_endpoint=None,
-                                              access_token_type=None,
-                                              **kwargs)
+        super().__init__(
+            auth_url=auth_url,
+            identity_provider=identity_provider,
+            protocol=protocol,
+            # intentionally set client_id, client_secret to empty since we
+            # don't need then
+            client_id='',  # nosec B106
+            client_secret='',  # nosec B106
+            access_token_type=access_token_type,
+            scope=scope,
+            access_token_endpoint=access_token_endpoint,
+            discovery_endpoint=discovery_endpoint,
+            trust_id=trust_id,
+            system_scope=system_scope,
+            domain_id=domain_id,
+            domain_name=domain_name,
+            project_id=project_id,
+            project_name=project_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name,
+            reauthenticate=reauthenticate,
+            include_catalog=include_catalog,
+        )
+        assert access_token is not None  # nosec B101
         self.access_token = access_token
 
-    def get_payload(self, session):
+    def get_payload(
+        self, session: ks_session.Session
+    ) -> dict[str, ty.Optional[str]]:
         """OidcAccessToken does not require a payload."""  # noqa: D403
         return {}
 
-    def get_unscoped_auth_ref(self, session):
+    def get_unscoped_auth_ref(
+        self, session: ks_session.Session
+    ) -> access.AccessInfoV3:
         """Authenticate with OpenID Connect and get back claims.
 
         We exchange the access token upon accessing the protected Keystone
@@ -537,7 +738,9 @@ class OidcAccessToken(_OidcBase):
         :rtype: :py:class:`keystoneauth1.access.AccessInfoV3`
         """
         response = self._get_keystone_token(session, self.access_token)
-        return access.create(resp=response)
+        access_info = access.create(resp=response)
+        assert isinstance(access_info, access.AccessInfoV3)  # nosec B101
+        return access_info
 
 
 class OidcDeviceAuthorization(_OidcBase):
@@ -546,13 +749,32 @@ class OidcDeviceAuthorization(_OidcBase):
     grant_type = "urn:ietf:params:oauth:grant-type:device_code"
     HEADER_X_FORM = {"Content-Type": "application/x-www-form-urlencoded"}
 
-    def __init__(self, auth_url, identity_provider, protocol,  # nosec
-                 client_id, client_secret=None,
-                 access_token_endpoint=None,
-                 device_authorization_endpoint=None,
-                 discovery_endpoint=None,
-                 code_challenge=None, code_challenge_method=None,
-                 **kwargs):
+    def __init__(
+        self,
+        auth_url: str,
+        identity_provider: str,
+        protocol: str,
+        client_id: str,
+        client_secret: ty.Optional[str] = None,
+        # access_token_type intentionally skipped
+        scope: str = 'openid profile',
+        access_token_endpoint: ty.Optional[str] = None,
+        discovery_endpoint: ty.Optional[str] = None,
+        device_authorization_endpoint: ty.Optional[str] = None,
+        code_challenge: ty.Optional[str] = None,
+        code_challenge_method: ty.Optional[str] = None,
+        *,
+        trust_id: ty.Optional[str] = None,
+        system_scope: ty.Optional[str] = None,
+        domain_id: ty.Optional[str] = None,
+        domain_name: ty.Optional[str] = None,
+        project_id: ty.Optional[str] = None,
+        project_name: ty.Optional[str] = None,
+        project_domain_id: ty.Optional[str] = None,
+        project_domain_name: ty.Optional[str] = None,
+        reauthenticate: bool = True,
+        include_catalog: bool = True,
+    ):
         """The OAuth 2.0 Device Authorization plugin expects the following.
 
         :param device_authorization_endpoint: OAuth 2.0 Device Authorization
@@ -571,18 +793,31 @@ class OidcDeviceAuthorization(_OidcBase):
         self.device_authorization_endpoint = device_authorization_endpoint
         self.code_challenge_method = code_challenge_method
 
-        super(OidcDeviceAuthorization, self).__init__(
+        super().__init__(
             auth_url=auth_url,
             identity_provider=identity_provider,
             protocol=protocol,
             client_id=client_id,
             client_secret=client_secret,
+            access_token_type=self.access_token_type,
+            scope=scope,
             access_token_endpoint=access_token_endpoint,
             discovery_endpoint=discovery_endpoint,
-            access_token_type=self.access_token_type,
-            **kwargs)
+            trust_id=trust_id,
+            system_scope=system_scope,
+            domain_id=domain_id,
+            domain_name=domain_name,
+            project_id=project_id,
+            project_name=project_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name,
+            reauthenticate=reauthenticate,
+            include_catalog=include_catalog,
+        )
 
-    def _get_device_authorization_endpoint(self, session):
+    def _get_device_authorization_endpoint(
+        self, session: ks_session.Session
+    ) -> ty.Optional[str]:
         """Get the endpoint for the OAuth 2.0 Device Authorization flow.
 
         This method will return the correct device authorization endpoint to
@@ -605,9 +840,9 @@ class OidcDeviceAuthorization(_OidcBase):
         endpoint = discovery.get("device_authorization_endpoint")
         if endpoint is None:
             raise exceptions.oidc.OidcDeviceAuthorizationEndpointNotFound()
-        return endpoint
+        return ty.cast(str, endpoint)
 
-    def _generate_pkce_verifier(self):
+    def _generate_pkce_verifier(self) -> str:
         """Generate PKCE verifier string as defined in RFC 7636."""
         raw_bytes = 42  # 32 is the minimum from the RFC, let's use a bit more
         _rand = os.urandom(raw_bytes)
@@ -615,10 +850,10 @@ class OidcDeviceAuthorization(_OidcBase):
         code_verifier = _rand_b64.rstrip('=')  # strip padding as RFC says
         return code_verifier
 
-    def _generate_pkce_challenge(self):
+    def _generate_pkce_challenge(self) -> ty.Optional[str]:
         """Generate PKCE challenge string as defined in RFC 7636."""
         if self.code_challenge_method not in ('plain', 'S256'):
-            raise exceptions.OidcGrantTypeMissmatch()
+            raise exceptions.OidcInvalidCodeChallengeMethod()
         self.code_verifier = self._generate_pkce_verifier()
 
         if self.code_challenge_method == 'plain':
@@ -626,11 +861,13 @@ class OidcDeviceAuthorization(_OidcBase):
         elif self.code_challenge_method == 'S256':
             _tmp = self.code_verifier.encode('ascii')
             _hash = hashlib.sha256(_tmp).digest()
-            _tmp = base64.urlsafe_b64encode(_hash).decode('ascii')
-            code_challenge = _tmp.rstrip('=')
-            return code_challenge
+            return base64.urlsafe_b64encode(_hash).decode('ascii').rstrip('=')
 
-    def get_payload(self, session):
+        return None
+
+    def get_payload(
+        self, session: ks_session.Session
+    ) -> dict[str, ty.Optional[str]]:
         """Get an authorization grant for the "device_code" grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -639,8 +876,13 @@ class OidcDeviceAuthorization(_OidcBase):
         :returns: a python dictionary containing the payload to be exchanged
         :rtype: dict
         """
-        device_authz_endpoint = \
-            self._get_device_authorization_endpoint(session)
+        payload: dict[str, ty.Optional[str]]
+        device_authz_endpoint = self._get_device_authorization_endpoint(
+            session
+        )
+
+        # TODO(stephenfin): This should be an error
+        assert device_authz_endpoint is not None  # nosec B101
 
         if self.client_secret:
             client_auth = (self.client_id, self.client_secret)
@@ -651,8 +893,9 @@ class OidcDeviceAuthorization(_OidcBase):
 
         if self.code_challenge_method:
             self.code_challenge = self._generate_pkce_challenge()
-            payload.setdefault('code_challenge_method',
-                               self.code_challenge_method)
+            payload.setdefault(
+                'code_challenge_method', self.code_challenge_method
+            )
             payload.setdefault('code_challenge', self.code_challenge)
         encoded_payload = urlparse.urlencode(payload)
 
@@ -660,19 +903,24 @@ class OidcDeviceAuthorization(_OidcBase):
             sanitized_payload = self._sanitize(payload)
             _logger.debug(
                 "Making OpenID-Connect authentication request to %s with "
-                "data %s", device_authz_endpoint, sanitized_payload
+                "data %s",
+                device_authz_endpoint,
+                sanitized_payload,
             )
-        op_response = session.post(device_authz_endpoint,
-                                   requests_auth=client_auth,
-                                   headers=self.HEADER_X_FORM,
-                                   data=encoded_payload,
-                                   log=False,
-                                   authenticated=False)
+        op_response = session.post(
+            device_authz_endpoint,
+            requests_auth=client_auth,
+            headers=self.HEADER_X_FORM,
+            data=encoded_payload,
+            log=False,
+            authenticated=False,
+        )
         if _logger.isEnabledFor(logging.DEBUG):
             sanitized_response = self._sanitize(op_response.json())
             _logger.debug(
                 "OpenID-Connect authentication response from %s is %s",
-                device_authz_endpoint, sanitized_response
+                device_authz_endpoint,
+                sanitized_response,
             )
 
         self.expires_in = int(op_response.json()["expires_in"])
@@ -681,28 +929,35 @@ class OidcDeviceAuthorization(_OidcBase):
         self.interval = int(op_response.json()["interval"])
         self.user_code = op_response.json()["user_code"]
         self.verification_uri = op_response.json()["verification_uri"]
-        self.verification_uri_complete = \
-            op_response.json()["verification_uri_complete"]
+        self.verification_uri_complete = op_response.json()[
+            "verification_uri_complete"
+        ]
 
         payload = {'device_code': self.device_code}
         if self.code_challenge_method:
             payload.setdefault('code_verifier', self.code_verifier)
         return payload
 
-    def _get_access_token(self, session, payload):
+    def _get_access_token(
+        self, session: ks_session.Session, payload: dict[str, ty.Optional[str]]
+    ) -> str:
         """Poll token endpoint for an access token.
 
         :param session: a session object to send out HTTP requests.
         :type session: keystoneauth1.session.Session
 
         :param payload: a dict containing various OpenID Connect values,
-                for example::
-                {'grant_type': 'urn:ietf:params:oauth:grant-type:device_code',
-                 'device_code': self.device_code}
+            for example::
+
+                {
+                    'grant_type': 'urn:ietf:params:oauth:grant-type:device_code',
+                    'device_code': self.device_code,
+                }
         :type payload: dict
         """
-        _logger.warning(f"To authenticate please go to: "
-                        f"{self.verification_uri_complete}")
+        _logger.warning(
+            f"To authenticate please go to: {self.verification_uri_complete}"
+        )
 
         if self.client_secret:
             client_auth = (self.client_id, self.client_secret)
@@ -720,21 +975,27 @@ class OidcDeviceAuthorization(_OidcBase):
                     _logger.debug(
                         "Making OpenID-Connect authentication request to %s "
                         "with data %s",
-                        access_token_endpoint, sanitized_payload
+                        access_token_endpoint,
+                        sanitized_payload,
                     )
-                op_response = session.post(access_token_endpoint,
-                                           requests_auth=client_auth,
-                                           data=encoded_payload,
-                                           headers=self.HEADER_X_FORM,
-                                           log=False,
-                                           authenticated=False)
+                op_response = session.post(
+                    access_token_endpoint,
+                    requests_auth=client_auth,
+                    data=encoded_payload,
+                    headers=self.HEADER_X_FORM,
+                    log=False,
+                    authenticated=False,
+                )
                 if _logger.isEnabledFor(logging.DEBUG):
                     sanitized_response = self._sanitize(op_response.json())
                     _logger.debug(
                         "OpenID-Connect authentication response from %s is %s",
-                        access_token_endpoint, sanitized_response
+                        access_token_endpoint,
+                        sanitized_response,
                     )
             except exceptions.http.BadRequest as exc:
+                if exc.response is None:
+                    raise
                 error = exc.response.json().get("error")
                 if error != "authorization_pending":
                     raise
@@ -746,4 +1007,5 @@ class OidcDeviceAuthorization(_OidcBase):
                 raise exceptions.oidc.OidcDeviceAuthorizationTimeOut()
 
         access_token = op_response.json()[self.access_token_type]
+        assert isinstance(access_token, str)  # nosec B101
         return access_token
