@@ -12,6 +12,8 @@
 
 import typing as ty
 
+import typing_extensions as ty_ext
+
 from keystoneauth1 import discover
 
 if ty.TYPE_CHECKING:
@@ -31,6 +33,13 @@ BaseAuthPluginT = ty.TypeVar(
 )
 
 
+class ConnectionParams(ty.TypedDict):
+    # https://github.com/python/typeshed/blob/24c78b9e0/stubs/requests/requests/sessions.pyi#L82
+    cert: ty_ext.NotRequired[ty.Union[str, ty.Tuple[str, str], None]]
+    # https://github.com/python/typeshed/blob/24c78b9e0/stubs/requests/requests/sessions.pyi#L108
+    verify: ty_ext.NotRequired[ty.Union[bool, str, None]]
+
+
 class BaseAuthPlugin:
     """The basic structure of an authentication plugin.
 
@@ -41,20 +50,15 @@ class BaseAuthPlugin:
     """
 
     def __init__(self) -> None:
-        self._discovery_cache: ty.Dict[str, discover.Discover] = {}
+        self._discovery_cache: dict[str, discover.Discover] = {}
 
-    def get_token(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Optional[str]:
+    def get_token(self, session: 'ks_session.Session') -> ty.Optional[str]:
         """Obtain a token.
 
         How the token is obtained is up to the plugin. If it is still valid
         it may be re-used, retrieved from cache or invoke an authentication
         request against a server.
 
-        There are no required kwargs. They are passed directly to the auth
-        plugin and they are implementation specific.
-
         Returning None will indicate that no token was able to be retrieved.
 
         This function is misplaced as it should only be required for auth
@@ -75,21 +79,21 @@ class BaseAuthPlugin:
         return None
 
     def get_auth_ref(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
+        self, session: 'ks_session.Session'
     ) -> ty.Optional['access.AccessInfo']:
         """Return the authentication reference of an auth plugin.
 
-        There are no required kwargs. They are passed directly to the auth
-        plugin and they are implementation specific.
-
         :param session: A session object to be used for communication
         :type session: keystoneauth1.session.session
+
+        :return: An access info object if supported.
+        :rtype: `keystoneauth1.access.AccessInfo` or None
         """
         return None
 
     def get_headers(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Optional[ty.Dict[str, str]]:
+        self, session: 'ks_session.Session'
+    ) -> ty.Optional[dict[str, str]]:
         """Fetch authentication headers for message.
 
         This is a more generalized replacement of the older get_token to allow
@@ -105,9 +109,6 @@ class BaseAuthPlugin:
         Subclasses should feel free to completely override this function to
         provide the headers that they want.
 
-        There are no required kwargs. They are passed directly to the auth
-        plugin and they are implementation specific.
-
         Returning None will indicate that no token was able to be retrieved and
         that authorization was a failure. Adding no authentication data can be
         achieved by returning an empty dictionary.
@@ -135,7 +136,7 @@ class BaseAuthPlugin:
         discover_versions: bool = True,
         **kwargs: ty.Any,
     ) -> ty.Optional[discover.EndpointData]:
-        """Return a valid endpoint data for a the service.
+        """Return a valid endpoint data for a service.
 
         :param session: A session object that can be used for communication.
         :type session: keystoneauth1.session.Session
@@ -172,7 +173,7 @@ class BaseAuthPlugin:
         *,
         endpoint_override: ty.Optional[str] = None,
         **kwargs: ty.Any,
-    ) -> ty.Optional[ty.Tuple[ty.Union[int, float], ...]]:
+    ) -> ty.Optional[tuple[ty.Union[int, float], ...]]:
         """Get the major API version from the endpoint.
 
         :param session: A session object that can be used for communication.
@@ -212,13 +213,10 @@ class BaseAuthPlugin:
     def get_all_version_data(
         self,
         session: 'ks_session.Session',
-        interface: str = 'public',
+        interface: ty.Union[str, list[str], None] = 'public',
         region_name: ty.Optional[str] = None,
         service_type: ty.Optional[str] = None,
-        **kwargs: ty.Any,
-    ) -> ty.Dict[
-        str, ty.Dict[str, ty.Dict[str, ty.List[discover.VersionData]]]
-    ]:
+    ) -> dict[str, dict[str, dict[str, list[discover.VersionData]]]]:
         """Get version data for all services in the catalog.
 
         :param session: A session object that can be used for communication.
@@ -234,6 +232,7 @@ class BaseAuthPlugin:
         :param string service_type:
             Limit the version data to a single service. (optional, defaults
             to None)
+
         :returns: A dictionary keyed by region_name with values containing
             dictionaries keyed by interface with values being a list of
             :class:`~keystoneauth1.discover.VersionData`.
@@ -257,6 +256,7 @@ class BaseAuthPlugin:
 
         :param session: The session object that the auth_plugin belongs to.
         :type session: keystoneauth1.session.Session
+        :param kwargs: Ignored.
 
         :returns: The base URL that will be used to talk to the required
                   service or None if not available.
@@ -270,16 +270,15 @@ class BaseAuthPlugin:
         return endpoint_data.url
 
     def get_connection_params(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Dict[str, ty.Any]:
+        self, session: 'ks_session.Session'
+    ) -> ConnectionParams:
         """Return any additional connection parameters required for the plugin.
 
         :param session: The session object that the auth_plugin belongs to.
         :type session: keystoneauth1.session.Session
 
-        :returns: Headers that are set to authenticate a message or None for
-                  failure. Note that when checking this value that the empty
-                  dict is a valid, non-failure response.
+        :returns: Parameters that are passed to the requests library. Only the
+                  ``cert`` and ``verify`` parameters may be returned.
         :rtype: dict
         """
         return {}
@@ -300,9 +299,7 @@ class BaseAuthPlugin:
         """
         return False
 
-    def get_user_id(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Optional[str]:
+    def get_user_id(self, session: 'ks_session.Session') -> ty.Optional[str]:
         """Return a unique user identifier of the plugin.
 
         Wherever possible the user id should be inferred from the token however
@@ -318,7 +315,7 @@ class BaseAuthPlugin:
         return None
 
     def get_project_id(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
+        self, session: 'ks_session.Session'
     ) -> ty.Optional[str]:
         """Return the project id that we are authenticated to.
 
@@ -335,7 +332,7 @@ class BaseAuthPlugin:
         return None
 
     def get_sp_auth_url(
-        self, session: 'ks_session.Session', sp_id: str, **kwargs: ty.Any
+        self, session: 'ks_session.Session', sp_id: str
     ) -> ty.Optional[str]:
         """Return auth_url from the Service Provider object.
 
@@ -352,7 +349,7 @@ class BaseAuthPlugin:
         return None
 
     def get_sp_url(
-        self, session: 'ks_session.Session', sp_id: str, **kwargs: ty.Any
+        self, session: 'ks_session.Session', sp_id: str
     ) -> ty.Optional[str]:
         """Return sp_url from the Service Provider object.
 
@@ -423,7 +420,10 @@ class FixedEndpointPlugin(BaseAuthPlugin):
         self.endpoint = endpoint
 
     def get_endpoint(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
+        self,
+        session: 'ks_session.Session',
+        endpoint_override: ty.Optional[str] = None,
+        **kwargs: ty.Any,
     ) -> ty.Optional[str]:
         """Return the supplied endpoint.
 
@@ -431,7 +431,7 @@ class FixedEndpointPlugin(BaseAuthPlugin):
         parameters passed to the plugin. endpoint_override overrides the
         endpoint specified when constructing the plugin.
         """
-        return kwargs.get('endpoint_override') or self.endpoint
+        return endpoint_override or self.endpoint
 
     def get_endpoint_data(
         self,
